﻿/* 
 * Capture Test NyARToolkitCSサンプルプログラム
 * --------------------------------------------------------------------------------
 * The MIT License
 * Copyright (c) 2008 nyatla
 * airmail(at)ebony.plala.or.jp
 * http://nyatla.jp/nyartoolkit/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 * 
 */
using System;
using System.Xml;
using System.Diagnostics;
using System.Text;
using System.IO;
using System.Reflection;
using System.Drawing;
using System.Collections.Generic;
using jp.nyatla.nyartoolkit.cs.core;
using NyARToolkitCSUtils.NyAR;
using NyARToolkitCSUtils.Direct3d;
using System.Windows.Forms;
using Microsoft.WindowsMobile.DirectX;

namespace ARDemo
{
    public class ResourceBuilder
    {

        public class TContentData
        {
            private ResourceBuilder _parent;
            private String _file;
            private String _patt;
            private String _act = null;
            /**<object file="" patt="">
             *  <action trigger="keypress" type="shell">http://nyatla.jp/</action>
             * </object>
             */
            public TContentData(ResourceBuilder i_parent, XmlNode i_content_node)
            {
                this._parent = i_parent;
                this._file = i_content_node.SelectSingleNode("@file").Value;
                this._patt = i_content_node.SelectSingleNode("@patt").Value;
                XmlNode act = i_content_node.SelectSingleNode("action");
                //<action>のパース
                if (act != null)
                {
                    XmlNode n;
                    n = act.SelectSingleNode("@trigger");
                    if (n == null || n.Value != "keypress")
                    {
                        throw new Exception("FAILED:@trigger!=\"keypress\"");
                    }
                    n = act.SelectSingleNode("@type");
                    if (n == null || n.Value != "shell")
                    {
                        throw new Exception("FAILED:@type!=\"shell\"");
                    }
                    this._act = act.InnerXml;
                }
                return;
            }

            public String file_path { get { return _parent.convertToFullPath(this._file); } }
            public String patt_path { get { return _parent.convertToFullPath(this._patt); } }
            public DsSingleARMarkerApplication.NyARCodeEx createNyARCode(D3dManager i_d3dmgr)
            {
                DsSingleARMarkerApplication.NyARCodeEx code = new DsSingleARMarkerApplication.NyARCodeEx(16, 16);
                code.loadARPattFromFile(this.patt_path);
                code.tag_dislay = ModelLoader.loadFromXML(i_d3dmgr, this.file_path);
                if (this._act != null)
                {
                    code.tag_action = new SimpleKeyAction(this._act);
                }
                return code;
            }
        };
        private List<TContentData> _content_list;
        public List<TContentData> content_list { get { return this._content_list; } }

        public const int IMGFMT_JPG = 0;
        public const int IMGFMT_GIF = 1;
        public const int IMGFMT_BMP = 2;
        public const int IMGFMT_PNG = 3;

        private String _capture_dir;
        public String capture_dir { get { return this._capture_dir; } }

        private int _capture_format;
        public int capture_format { get { return this._capture_format; } }

        private String _root_path;
        public String root_path { get { return this._root_path; } }

        //コンテンツは今の所１個に限定
        private String _cpara_file;//data\\camera_para.dat";
        private bool _cvertical;//false
        public bool cvertical { get { return this._cvertical; } }
        private int _background_type;
        private Size _cap_size = new Size();
        public NyARParam createNyARParam()
        {
            NyARParam param;
            param = new NyARParam();
            param.loadARParamFromFile(this._root_path + "\\" + this._cpara_file);
            param.changeScreenSize(this._cap_size.Width, this._cap_size.Height);
            return param;
        }
        private String convertToFullPath(String i_path)
        {
            if (i_path[0] != '\\')
            {
                return this._root_path + "\\" + i_path;
            }
            return i_path;
        }
        public WmCapture createWmCapture()
        {
            WmCapture result = new WmCapture(this._cap_size);
            return result;
        }
        public D3dManager createD3dManager(Form i_form)
        {
            D3dManager result;
            result = new D3dManager(i_form, this._cap_size, this._background_type);
            return result;

        }
        public String getCameraParamFilePath()
        {
            return this._root_path + "\\" + this._cpara_file;
        }
        public ResourceBuilder()
        {
            //ルートパスの取得
            this._root_path = Path.GetDirectoryName(Assembly.GetExecutingAssembly().GetModules()[0].FullyQualifiedName);
            //設定の読み出し

            XmlDocument dom = new XmlDocument();
            dom.Load(this._root_path + "\\setting.xml");
            if (dom.SelectSingleNode("/root/version").InnerText != "ARDemo/0.1")
            {
                throw new Exception("setting::設定ファイルのバージョンが違います？");
            }

            //configセクション
            XmlNode config_node = dom.SelectSingleNode("/root/config");
            this._cap_size.Width = int.Parse(config_node.SelectSingleNode("camera/@width").Value);
            this._cap_size.Height = int.Parse(config_node.SelectSingleNode("camera/@height").Value);
            this._cpara_file = config_node.SelectSingleNode("camera/@file").Value;
            this._cvertical = bool.Parse(config_node.SelectSingleNode("camera/@vertical").Value);

            String bgmode = config_node.SelectSingleNode("direct3d/@background").Value;
            this._background_type = parse_direct3d_background(bgmode);


            this._capture_dir = config_node.SelectSingleNode("capture/@directory").Value;
            this._capture_format = parse_capture_format(config_node.SelectSingleNode("capture/@format").Value);

            //コンテンツセクション
            this._content_list = new List<TContentData>();
            foreach (XmlNode n in dom.SelectNodes("/root/contents/object"))
            {
                this._content_list.Add(new TContentData(this, n));
            }
            dom = null;
            return;
        }
        private int parse_capture_format(String i_s)
        {
            switch (i_s)
            {
                case "jpg": return IMGFMT_JPG;
                case "png": return IMGFMT_PNG;
                case "bmp": return IMGFMT_BMP;
                case "gif": return IMGFMT_GIF;
                default:
                    throw new Exception("formatの値が不正です。");
            }
        }

        private int parse_direct3d_background(String i_s)
        {
            switch (i_s)
            {
                case "texture":
                    return D3dManager.BGMODE_TEXTURE;
                case "surface":
                    return D3dManager.BGMODE_SURFACE;
                default:
                    throw new Exception("direct3d@backgroundの値が不正です。");
            }
        }
    }
}
