﻿/* 
 * Capture Test NyARToolkitCSサンプルプログラム
 * --------------------------------------------------------------------------------
 * The MIT License
 * Copyright (c) 2008 nyatla
 * airmail(at)ebony.plala.or.jp
 * http://nyatla.jp/nyartoolkit/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 * 
 */
using System;
using System.IO;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.Threading;
using System.Windows.Forms;
using System.Xml;
using System.Media;
using Microsoft.WindowsMobile.DirectX;
using Microsoft.WindowsMobile.DirectX.Direct3D;
using NyARToolkitCSUtils.NyAR;
using NyARToolkitCSUtils.Direct3d;
using jp.nyatla.nyartoolkit.cs;
using jp.nyatla.nyartoolkit.cs.core;
using jp.nyatla.nyartoolkit.cs.detector;
using jp.nyatla.nyartoolkit.cs.sandbox.quadx2;
using jp.nyatla.nyartoolkit.cs.sandbox.x2;
using jp.nyatla.cs.NyWMCapture;
using Microsoft.Samples.MD3DM;
using jp.nyatla.nyartoolkit.cs.processor;

namespace ARDemo
{



    public class DsSingleARMarkerApplication : IDisposable, INySampleCB
    {
        public interface IDisplayObject : IDisposable
        {
            void onOpen(DsSingleARMarkerApplication i_sender);
            bool onClose(DsSingleARMarkerApplication i_sender);
            void onDisplay(DsSingleARMarkerApplication i_sender, Device i_dev, ref Matrix i_transmat);
            //void onRequest(/*actionManager*/);
        }
        public interface IActionHandler
        {
            void onKey(DsSingleARMarkerApplication i_sender);
        }
        private class MyARProcesser : SingleARMarkerProcesser_X2
        {
            private Object parent;
            public MyARProcesser(Object i_parent, NyARParam i_param,int i_src_type)
            {
                initInstance(i_param, i_src_type);
                this.parent = i_parent;
                return;
            }
            protected override void onEnterHandler(int i_code)
            {
                DsSingleARMarkerApplication parent = (DsSingleARMarkerApplication)this.parent;
                parent._current_object = parent._items[i_code].tag_dislay;
                parent._current_object.onOpen(parent);
            }
            protected override void onUpdateHandler(NyARSquare i_square, NyARTransMatResult i_result)
            {
                DsSingleARMarkerApplication parent = (DsSingleARMarkerApplication)this.parent;
                parent._d3dmgr.utils.toD3dMatrix(i_result, ref  parent._transmat);
            }
            protected override void onLeaveHandler()
            {
                DsSingleARMarkerApplication parent = (DsSingleARMarkerApplication)this.parent;
                parent._current_object.onClose(parent);
                parent._current_object = null;
            }
        }
        public class NyARCodeEx : NyARCode
        {
            public NyARCodeEx(int i_w, int i_h)
                : base(i_w, i_h)
            { }
            public IDisplayObject tag_dislay;
            public IActionHandler tag_action;
        };
        private List<NyARCodeEx> _items = new List<NyARCodeEx>();
        private MyARProcesser _processer;
        //Direct3d
        private ID3dBackground _d3d_back_ground;
        private IDisplayObject _current_object;
        private WmCapture _capture;
        private D3dManager _d3dmgr;
        private Matrix _transmat;
        //表示アイテムのリスト
        //        private List<NyARCode> _codes = new List<NyARCode>();
        //        private List<IDisplayObject> _disps = new List<IDisplayObject>();
        private int _marker_resolution;
        private double _marker_width;
        private DsRGB565Raster _raster;
        private bool _capture_vertical;
        protected D3dManager d3dmgr { get { return this._d3dmgr; } }

        public DsSingleARMarkerApplication(D3dManager i_d3d, WmCapture i_capture, bool i_capture_vertical, NyARParam i_param, int i_marker_resolution, double i_marker_width)
        {
            NyMath.initialize();
            Matrix tmp = new Matrix();
            i_d3d.utils.toCameraFrustumRH(i_param, ref tmp);
            i_d3d.d3d_device.Transform.Projection = tmp;
            this._capture = i_capture;
            this._capture.setOnSample(this);
            this._capture_vertical = i_capture_vertical;
            this._d3dmgr = i_d3d;

            this._d3dmgr.d3d_device.DeviceLost += new EventHandler(OnDeviceLost);
            this._d3dmgr.d3d_device.DeviceReset += new EventHandler(OnDeviceReset);

            NyARIntSize size = i_param.getScreenSize();
            this._raster = new DsRGB565Raster(size.w, size.h);
            this._processer = new MyARProcesser(this, i_param, this._raster.getBufferReader().getBufferType());
            this._d3d_back_ground = i_d3d.createARBackGround(i_capture_vertical);
            this._marker_width = i_marker_width;
            this._marker_resolution = i_marker_resolution;
            return;
        }
        public void registerDisplayObject(NyARCodeEx i_arcodeex)
        {
            this._items.Add(i_arcodeex);
            return;
        }
        public void start()
        {
            this._processer.setARCodeTable(this._items.ToArray(), this._marker_resolution, this._marker_width);
            _capture.start();
            return;
        }
        public void stop()
        {
            this._capture.stop();
            //未認識状態に通常遷移
            this._processer.reset(false);
            return;
        }
        public void present()
        {
            int st = System.Environment.TickCount;
 
            //ライト有効
            lock (this)
            {
                // 3Dオブジェクトの描画はここから
                Device dev = this._d3dmgr.d3d_device;
                dev.BeginScene();
                this._d3d_back_ground.drawBackGround();

                this._d3dmgr.d3d_device.RenderState.CullMode = Cull.Clockwise;

                this._d3dmgr.d3d_device.Clear(ClearFlags.ZBuffer, System.Drawing.Color.Blue, 1.0f, 0);
                //現在の認識状況を見てイベントコール
                if (this._current_object != null)
                {
                    this._current_object.onDisplay(this, this._d3dmgr.d3d_device, ref this._transmat);
                }

                // 描画はここまで
                this._d3dmgr.d3d_device.EndScene();
                // 実際のディスプレイに描画
                this._d3dmgr.d3d_device.Present();
            }

            int tw = System.Environment.TickCount - st;
//            Debug.WriteLine("PWait=" + tw);

            if (tw < 100 && tw >= 0)
            {
                Thread.Sleep(tw);//10fpsくらいを目標に
            }
            else
            {
                Thread.Sleep(0);//10fpsくらいを目標に
            }

            return;
        }

        public void keyPressHandler(object sender, KeyPressEventArgs e)
        {
            lock (this)
            {
                if (this._current_object == null)
                {
                    return;
                }
                foreach (NyARCodeEx n in this._items)
                {
                    if (n.tag_dislay != this._current_object)
                    {
                        continue;
                    }
                    if (n.tag_action == null)
                    {
                        continue;
                    }
                    n.tag_action.onKey(this);
                }
            }
        }
        public int OnSample(INySample i_sample)
        {

            lock (this)
            {
                IntPtr data = i_sample.GetData();
                this._raster.setBuffer(data, this._capture_vertical);
                //テクスチャ内容を更新

                this._d3d_back_ground.CopyFromRaster(this._raster);
                //検出プロセッサを実行
//                int st = System.Environment.TickCount;
                this._processer.detectMarker(this._raster);
//                int tw = System.Environment.TickCount - st;
//                Debug.WriteLine("CWait=" + tw);
            }
            Thread.Sleep(0);

            return 0;
        }
        // リソースの破棄をするために呼ばれる
        public void Dispose()
        {
            if (this._capture != null)
            {
                this._capture.Dispose();
                this._capture = null;
            }
            for (int i = 0; i < this._items.Count; i++)
            {
                this._items[i].tag_dislay.Dispose();
            }
            if (this._d3d_back_ground != null)
            {
                this._d3d_back_ground.Dispose();
            }
            // Direct3D デバイスのリソース解放
            if (this._d3dmgr != null)
            {
                this._d3dmgr.Dispose();
            }
            return;
        }
        private void OnDeviceLost(object sender, EventArgs e)
        {
            MessageBox.Show("OnDeviceLost");
            return;
        }
        private void OnDeviceReset(object sender, EventArgs e)
        {
            MessageBox.Show("OnDeviceReset");
            return;
        }
    }





    public partial class SimpleLiteD3d : DsSingleARMarkerApplication
    {
        public String version="Uma2Mobile 0.3.0 (c)2008-2009 nyatla.\r\nhttp://nyatla.jp/\r\nMikuモデル ファイル保管庫＠七葉\r\nhttp://nanoha.kirara.st/3dcg/file/";
        //NyAR
        private String _capture_image_dir;
        private int _capture_image_format;

        public SimpleLiteD3d(Uma2Mobile topLevelForm, ResourceBuilder i_resource) :
            base(
                 i_resource.createD3dManager(topLevelForm),
                 i_resource.createWmCapture(),
                 i_resource.cvertical,
                 i_resource.createNyARParam(),
                 16, 80.0)
        {
            //キャプチャ情報の保管
            this._capture_image_dir = i_resource.capture_dir;
            this._capture_image_format = i_resource.capture_format;
            foreach (ResourceBuilder.TContentData n in i_resource.content_list)
            {
                //NyARCodeExはタグ情報をつけたNyARCodeオブジェクトです。

                this.registerDisplayObject(n.createNyARCode(this.d3dmgr));
            }
            return;
        }
        //ファイルを保存する。
        public void saveImage()
        {
            System.Drawing.Imaging.ImageFormat fmt;
            String file_name = this._capture_image_dir + "\\UM_" + Convert.ToString(DateTime.Now.Ticks, 16);
            lock (this)
            {
                switch (this._capture_image_format)
                {
                    case ResourceBuilder.IMGFMT_BMP:
                        file_name += ".bmp";
                        fmt = System.Drawing.Imaging.ImageFormat.Bmp;
                        break;
                    case ResourceBuilder.IMGFMT_GIF:
                        file_name += ".gif";
                        fmt = System.Drawing.Imaging.ImageFormat.Gif;
                        break;
                    case ResourceBuilder.IMGFMT_JPG:
                        file_name += ".jpeg";
                        fmt = System.Drawing.Imaging.ImageFormat.Jpeg;
                        break;
                    case ResourceBuilder.IMGFMT_PNG:
                        file_name += ".png";
                        fmt = System.Drawing.Imaging.ImageFormat.Png;
                        break;
                    default:
                        throw new Exception("未知のフォーマット");
                }
                try
                {
                    SystemSounds.Hand.Play();
                    this.d3dmgr.saveFrontSurfaceToFile(file_name, fmt);
                }
                catch (Exception e)
                {
                    MessageBox.Show("ERROR:saveImage", e.Message);
                }
            }
            return;
        }
    }
}
