﻿/*
 * NyARToolkitCSUtils NyARToolkit for C#
 * SimpleLiteDirect3d for WindowsMobile
 * 
 * (c)2008 nyatla
 * airmail(at)ebony.plala.or.jp
 * http://nyatla.jp/
 */
using System;
using System.IO;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.Threading;
using System.Windows.Forms;
using System.Reflection;
using Microsoft.WindowsMobile.DirectX;
using Microsoft.WindowsMobile.DirectX.Direct3D;
using NyARToolkitCSUtils.NyAR;
using NyARToolkitCSUtils.Direct3d;
using jp.nyatla.nyartoolkit.cs;
using jp.nyatla.nyartoolkit.cs.core;
using jp.nyatla.nyartoolkit.cs.detector;
using jp.nyatla.cs.NyWMCapture;
using Microsoft.Samples.MD3DM;
using jp.nyatla.nyartoolkit.cs.sandbox.quadx2;

namespace SimpleLiteDirect3d.WindowsMobile5
{
    //テクスチャを共有するためのメッシュセット
    public class MeshData
    {
        private Texture[]  m_textures = null;
        private Material[] m_meshMaterials=null;
        private Mesh m_mesh;
        private Device m_ref_dev;

        public MeshData(Device i_dev, String i_dir, String i_mesh_filename, Texture[] i_textures)
        {
            this.m_ref_dev = i_dev;
            string[] textureFilenames = null;
            // Ｘファイルを読み込んでメッシュを作成する
            this.m_mesh = MeshLoader.LoadMesh(i_dev,
                new FileStream(i_dir + i_mesh_filename, FileMode.Open, FileAccess.Read), MeshFlags.SystemMemory,
                out this.m_meshMaterials, out textureFilenames);
            if(i_textures==null){
                this.m_textures = new Texture[this.m_meshMaterials.Length];
                for (int i = 0; i < this.m_meshMaterials.Length; i++)
                {
                    // Set the ambient color for the material
                    this.m_meshMaterials[i].Ambient = this.m_meshMaterials[i].Diffuse;
                    // Create the texture
                    this.m_textures[i] = TextureLoader.FromStream(i_dev,
                        new FileStream(i_dir + textureFilenames[i], FileMode.Open, FileAccess.Read));
                }
            }else{
                for (int i = 0; i < this.m_meshMaterials.Length; i++)
                {
                    // Set the ambient color for the material
                    this.m_meshMaterials[i].Ambient = this.m_meshMaterials[i].Diffuse;
                }
            }
        }
        public void Draw()
        {
            for (int i = 0; i < this.m_meshMaterials.Length; i++)
            {
                // マテリアルをセット
                this.m_ref_dev.Material = this.m_meshMaterials[i];
                // テクスチャーのセット
                this.m_ref_dev.SetTexture(0, this.m_textures[i]);
                // 描画
                this.m_mesh.DrawSubset(i);
            }
        }
    }

    public partial class SimpleLiteD3d : IDisposable,INySampleCB
    {
        public int fps_x_100 = 0;
        private MeshData[] m_mesh_data=new MeshData[2];

        DeviceAdapter m_dev_adapter;
        /*  exeファイルのあるディレクトリからの相対パス
         *  デバックの時は、exeファイルはprogram files\[project name]に配置されるみたい。
         */
        private const String AR_CODE_FILE = "data\\patt.hiro";
        private const String AR_CAMERA_FILE = "data\\camera_para.dat";

        //NyAR
        private NyARSingleDetectMarker_Quad m_ar;
        private DsRGB565Raster m_raster;
        //背景テクスチャ
        private NyARSurface_RGB565 m_surface;
        //変数作りおき
        private Rectangle m_dest_rect;
        private NyARD3dUtil _utils;

        /// Direct3D デバイス
        private Device _device = null;
        private bool m_skip_capture=false;
        private Brush _fps_brush = new SolidBrush(Color.Red);
        private System.Drawing.Font _fps_font = new System.Drawing.Font(FontFamily.GenericSerif, 9.0f, FontStyle.Bold);
        
        /* 非同期イベントハンドラ
          * CaptureDeviceからのイベントをハンドリングして、バッファとテクスチャを更新する。
          */
        public int OnSample(INySample i_sample)
        {
            //キャプチャをスキップするならここでおしまい。
            if (m_skip_capture)
            {
                //キャプチャのスキップ要求があるなら長めにスリープ
                Thread.Sleep(0);
                return 0;
            }
            lock (this)
            {

                IntPtr data=i_sample.GetData();
                this.m_raster.setBuffer(data,this.m_dev_adapter.IsTurnCapVertical);
                //テクスチャ内容を更新
                this.m_surface.CopyFromIntPtr(i_sample, this.m_dev_adapter.IsTurnCapVertical);
                Graphics gs = this.m_surface.d3d_surface.GetGraphics();
                gs.DrawString(fps_x_100 / 100 + "." + fps_x_100 % 100 + "fps", _fps_font, _fps_brush, 0, 0);
                this.m_surface.d3d_surface.ReleaseGraphics();

                this.m_skip_capture = true;
            }
            Thread.Sleep(0);
            return 0;
        }
        /* キャプチャを開始する関数
         */
        public void StartCap()
        {
            this.m_dev_adapter.CaptureIf.Start();
        }
        /* キャプチャを停止する関数
         */
        public void StopCap()
        {
            this.m_dev_adapter.CaptureIf.Stop();
        }


        /* Direct3Dデバイスを準備する関数
         */
        private Device PrepareD3dDevice(Control i_window)
        {
            PresentParameters pp = new PresentParameters();

            // ウインドウモードなら true、フルスクリーンモードなら false を指定
            pp.Windowed = true;
            // スワップとりあえずDiscardを指定。
            pp.SwapEffect = SwapEffect.Discard;
            pp.EnableAutoDepthStencil = true;
            pp.AutoDepthStencilFormat = DepthFormat.D16;
            pp.BackBufferCount = 1;
            pp.BackBufferFormat = Format.R5G6B5;
            Device dev = new Device(0, DeviceType.Default, i_window.Handle, CreateFlags.None, pp);
            return dev;
        }

        public bool InitializeApplication(NyARToolkitCS topLevelForm, DeviceAdapter i_adapter)
        {
            String current_path=Path.GetDirectoryName(Assembly.GetExecutingAssembly().GetModules()[0].FullyQualifiedName);
            this.m_dev_adapter = i_adapter;
            Size cap_size = i_adapter.CaptureSize;
            
            //ARの設定

            //Direct3d用のユーティリティ準備
            this._utils = new NyARD3dUtil();

            //AR用カメラパラメタファイルをロードして設定
            NyARParam ap = new NyARParam();
            ap.loadARParamFromFile(current_path + "\\" + AR_CAMERA_FILE);
            ap.changeScreenSize(cap_size.Width,cap_size.Height);

            //AR用のパターンコードを読み出し	
            NyARCode code = new NyARCode(16, 16);
            code.loadARPattFromFile(current_path+"\\"+AR_CODE_FILE);

            //１パターンのみを追跡するクラスを作成
            this.m_ar = new NyARSingleDetectMarker_Quad(ap, code, 80.0);

            //計算モードの設定
            this.m_ar.setContinueMode(false);

            
            //3dデバイスを準備する
            this._device = PrepareD3dDevice(topLevelForm);
            this._device.RenderState.ZBufferEnable = true;
            this._device.RenderState.TexturePerspective = true;

            //ビューポート計算とスケールの計算
            Viewport vp=this.m_dev_adapter.D3dViewport;
            this._device.Viewport = vp;

            this.m_dest_rect = new Rectangle(vp.X, vp.Y, vp.Width, vp.Height);

            //カメラProjectionの設定
            Matrix tmp = new Matrix();
            this._utils.toCameraFrustumRH(ap, ref tmp);
            this._device.Transform.Projection = tmp;

            // ビュー変換の設定(左手座標系ビュー行列で設定する)
            // 0,0,0から、Z+方向を向いて、上方向がY軸
            this._device.Transform.View = Matrix.LookAtLH(
                new Vector3(0.0f, 0.0f, 0.0f), new Vector3(0.0f, 0.0f, 1.0f), new Vector3(0.0f, 1.0f, 0.0f));

            this.m_mesh_data[0] = new MeshData(this._device, current_path + "\\miku\\", "miku1.md3dm", null);
            this.m_mesh_data[1] = new MeshData(this._device, current_path + "\\miku\\", "miku2.md3dm", null);

            this._device.RenderState.Lighting = false;

            //ARラスタを作る(DirectShowキャプチャ仕様)。
            this.m_raster = new DsRGB565Raster(cap_size.Width, cap_size.Height);
            //背景テクスチャを作成
            this.m_surface = new NyARSurface_RGB565(this._device, cap_size.Width, cap_size.Height);
            return true;
        }
        Vector3 m_scale= new Vector3(20f, 20f, 20f);
        Matrix trans_matrix = new Matrix();
        private NyARTransMatResult __MainLoop_nyar_transmat = new NyARTransMatResult();
        int m_bind_lv = 0;//5回ミスるまでは最後のマトリクスを引っ張る。
        //メインループ処理
        public void MainLoop()
        {
            NyARTransMatResult trans_result = this.__MainLoop_nyar_transmat;

            //ARの計算
            
            bool is_marker_enable;
            lock (this)
            {
                //マーカーは見つかったかな？
                is_marker_enable = this.m_ar.detectMarkerLite(this.m_raster, 110);
                if (is_marker_enable)
                {
                    //あればMatrixを更新
                    this.m_ar.getTransmationMatrix(trans_result);
                    this._utils.toD3dMatrix(trans_result, ref trans_matrix);

                    //一致度を調べて、マーカーが使えるか判定
                    if (this.m_ar.getConfidence() > 0.3)
                    {
                        this.m_bind_lv = 5;
                    }
                }else{
                    if (this.m_bind_lv > 0)
                    {
                        this.m_bind_lv--;
                    }
                }
                //Zバッファクリア
                this._device.Clear(ClearFlags.ZBuffer,System.Drawing.Color.Blue, 1.0f, 0);
                

                //背景描画
                Surface dest_surface=this._device.GetBackBuffer(0, BackBufferType.Mono);
                this._device.StretchRectangle(this.m_surface.d3d_surface, this.m_surface.d3d_surface_rect, dest_surface, this.m_dest_rect, TextureFilter.Point);

                // 3Dオブジェクトの描画はここから
                this._device.BeginScene();

                //レベル0以上なら描画
                if (this.m_bind_lv>0)
                {
                    //モデル表示
                    Matrix transform_mat2 = Matrix.Scaling(this.m_scale);
                    transform_mat2 *= Matrix.RotationX((float)Math.PI * 1 / 2);
                    //変換行列を掛ける
                    transform_mat2 *= trans_matrix;

                    // 計算したマトリックスで座標変換
                    this._device.SetTransform(TransformType.World, transform_mat2);
                    //ちょっとしたギミック
                    if (trans_matrix.M43 > 120 && trans_matrix.M43 < 200)
                    {

                        this.m_mesh_data[1].Draw();
                    }
                    else
                    {
                        this.m_mesh_data[0].Draw();
                    }
                }

                // 描画はここまで
                this._device.EndScene();

                // 実際のディスプレイに描画
                this._device.Present();
                this.m_skip_capture = false;//スキップ解除
            }
        }

        // リソースの破棄をするために呼ばれる
        public void Dispose()
        {
 
            // Direct3D デバイスのリソース解放
            if (this._device != null)
            {
                this._device.Dispose();
            }
        }
    }
}
