﻿using System;
using System.Collections.Generic;
using System.Text;
using jp.nyatla.nyartoolkit.cs.core;
using Microsoft.WindowsMobile.DirectX.Direct3D;
using Microsoft.WindowsMobile.DirectX;
using NyARToolkitCSUtils.Direct3d;
using System.Xml;
namespace ARDemo
{


    public class AnimationModel : DsSingleARMarkerApplication.IDisplayObject
    {
        private MeshData[] _mesh_table;
        private XYZAttr _posinfo;
        private uint _draw_counter = 0;
        private MeshData[] _mesh_data;
        private Matrix _basemat;
        private bool _light = false;
        //メッシュデータのリストと、名前インデクスを作ります。
        private static MeshData[] CreateMeshList(Device i_dev, String i_dir, XmlNodeList i_node, out String[] o_name)
        {
            List<MeshData> m = new List<MeshData>();
            List<string> s = new List<string>();
            String temp_name;

            //重複がないように名前とメッシュリストを作成
            temp_name = i_node[0].Value;
            MeshData key_mesh = new MeshData(i_dev, i_dir + "\\", temp_name);
            s.Add(temp_name);
            m.Add(key_mesh);
            for (int i = 1; i < i_node.Count; i++)
            {
                temp_name = i_node[i].Value;
                int i2;
                for (i2 = s.Count - 1; i2 >= 0; i2--)
                {
                    if (temp_name == s[i2])
                    {
                        break;
                    }
                }
                if (i2 >= 0)
                {
                    continue;
                }
                s.Add(temp_name);
                m.Add(new MeshData(i_dev, i_dir + "\\", temp_name, key_mesh));
            }
            o_name = s.ToArray();
            return m.ToArray();
        }

        public AnimationModel(D3dManager i_d3dmgr, XmlDocument i_dom, String i_xml_dir)
        {
            Device dev = i_d3dmgr.d3d_device;

            if (i_dom.SelectSingleNode("/root/version").InnerText != "AnimationModel/0.1")
            {
                throw new Exception("object::設定ファイルのバージョンが違います？");
            }
            //<md3dmset>からメッシュリストを作る。
            float sc = XmlHelper.getAttrValue(i_dom, "/root/md3dmset/@scale", 1.0f);
            this._basemat = Matrix.Scaling(sc, sc, sc);
            //ライトは？
            this._light = XmlHelper.getAttrValue(i_dom, "/root/md3dmset/@light", false);

            //position
            this._posinfo = XYZAttr.createInstance(i_dom.SelectSingleNode("/root/position"));
            //rotate
            XYZAttr rotate = XYZAttr.createInstance(i_dom.SelectSingleNode("/root/rotate"));
            if (rotate != null)
            {
                this._basemat = this._basemat * Matrix.RotationYawPitchRoll((float)(rotate.x * Math.PI / 180), (float)(rotate.y * Math.PI / 180), (float)(rotate.z * Math.PI / 180));
            }


            //md3dmsetから名前インデックスとメッシュリストを作る。
            XmlNodeList md3dm_nodes = i_dom.SelectNodes("/root/md3dmset/md3dm/@file");
            if (md3dm_nodes.Count < 1)
            {
                throw new Exception("MeshSet:has not md3dm node.");
            }
            //メッシュとメッシュ名のテーブルを作成
            String[] mesh_name;
            this._mesh_table = CreateMeshList(dev, i_xml_dir, md3dm_nodes, out mesh_name);
            //メッシュデータを作成
            this._mesh_data = new MeshData[md3dm_nodes.Count];
            for (int i = 0; i < md3dm_nodes.Count; i++)
            {
                this._mesh_data[i] = null;
                for (int i2 = this._mesh_table.Length - 1; i2 >= 0; i2--)
                {
                    if (mesh_name[i2] == md3dm_nodes[i].Value)
                    {
                        this._mesh_data[i] = this._mesh_table[i2];
                        break;
                    }
                }
                if (this._mesh_data[i] == null)
                {
                    throw new Exception("MeshSet:Invalid md3dm index.");
                }
            }
            return;
        }
        public void onOpen(DsSingleARMarkerApplication i_sender)
        {
        }
        public bool onClose(DsSingleARMarkerApplication i_sender)
        {
            //常にtrueを返してね。
            return true;
        }
        public void onDisplay(DsSingleARMarkerApplication i_sender, Device i_dev, ref Matrix i_transmat)
        {
            Matrix transform_mat2 = this._basemat;
            //position移動
            if (this._posinfo != null)
            {
                transform_mat2 *= Matrix.Translation(this._posinfo.x, this._posinfo.y, this._posinfo.z);
            }
            transform_mat2 *= Matrix.RotationX((float)Math.PI * 1 / 2);
            //変換行列を掛ける
            transform_mat2 *= i_transmat;
            // 計算したマトリックスで座標変換
            i_dev.SetTransform(TransformType.World, transform_mat2);

            //ライト
            bool old_l = i_dev.RenderState.Lighting;
            i_dev.RenderState.Lighting = this._light;

            this._mesh_data[this._draw_counter % this._mesh_data.Length].Draw();

            i_dev.RenderState.Lighting = old_l;

            this._draw_counter++;
            return;
        }

        public void Dispose()
        {
            for (int i = 0; i < this._mesh_table.Length - 1; i++)
            {
                if (this._mesh_table[i] != null)
                {
                    this._mesh_table[i] = null;
                }
            }
            return;
        }
    }
}