﻿using System;
using System.Collections.Generic;
using System.Text;
using jp.nyatla.nyartoolkit.cs.core;
using Microsoft.WindowsMobile.DirectX.Direct3D;
using Microsoft.WindowsMobile.DirectX;
using NyARToolkitCSUtils.Direct3d;
using System.Xml;
namespace ARDemo
{
    public class StaticModel : DsSingleARMarkerApplication.IDisplayObject
    {


        private RotateNode _anrotinfo;
        private XYZAttr _posinfo;

        private uint _draw_counter = 0;
        private IMeshData _mesh_data;
        private Matrix _basemat;
        private bool _light;
        public StaticModel(D3dManager i_d3dmgr, XmlDocument i_dom, String i_xml_dir)
        {
            Device dev = i_d3dmgr.d3d_device;

            //ファイル名の確定
            if (i_dom.SelectSingleNode("/root/version").InnerText != "StaticModel/0.1")
            {
                throw new Exception("object::設定ファイルのバージョンが違います？");
            }
            //animationからアニメーションルールを作る。

            //animation/rotationのデータを取得
            this._anrotinfo = RotateNode.createInstance(i_dom.SelectSingleNode("/root/animation/rotate"));

            //position
            this._posinfo = XYZAttr.createInstance(i_dom.SelectSingleNode("/root/position"));
            //rotate
            XYZAttr rotate = XYZAttr.createInstance(i_dom.SelectSingleNode("/root/rotate"));
            //md3dmからメッシュを作る。
            //basemat=scale*rotate
            float sc = XmlHelper.getAttrValue(i_dom, "/root/md3dm/@scale", 1.0f);
            this._basemat = Matrix.Scaling(sc, sc, sc);
            if (rotate != null)
            {
                this._basemat = this._basemat * Matrix.RotationYawPitchRoll((float)(rotate.x * Math.PI / 180), (float)(rotate.y * Math.PI / 180), (float)(rotate.z * Math.PI / 180));
            }
            //light
            this._light = XmlHelper.getAttrValue(i_dom, "/root/md3dm/@light", false);

            //@fileからメッシュリストを作る。

            String mesh_name = i_dom.SelectSingleNode("/root/md3dm/@file").Value;
            this._mesh_data = new MeshData(dev, i_xml_dir + "\\", mesh_name);

            return;
        }
        public void onOpen(DsSingleARMarkerApplication i_sender)
        {
        }
        public bool onClose(DsSingleARMarkerApplication i_sender)
        {
            //常にtrueを返してね。
            return true;
        }
        public void onDisplay(DsSingleARMarkerApplication i_sender, Device i_dev, ref Matrix i_transmat)
        {
            Matrix transform_mat2 = this._basemat;
            //rotationが指定されていれば、rotationを反映
            if (this._anrotinfo != null)
            {
                this._anrotinfo.multMatrix(ref transform_mat2);
            }
            //position移動
            if (this._posinfo != null)
            {
                transform_mat2 *= Matrix.Translation(this._posinfo.x, this._posinfo.y, this._posinfo.z);
            }
            //位相ずれを修正
            transform_mat2 *= Matrix.RotationX((float)Math.PI * 1 / 2);
            //変換行列を掛ける
            transform_mat2 *= i_transmat;
            // 計算したマトリックスで座標変換
            i_dev.SetTransform(TransformType.World, transform_mat2);
            //ライト
            bool old_l = i_dev.RenderState.Lighting;
            i_dev.RenderState.Lighting = this._light;

            this._mesh_data.Draw();
            i_dev.RenderState.Lighting = old_l;

            //クロックをインクリメント
            this._draw_counter++;
            return;
        }

        public void Dispose()
        {
            if (this._mesh_data != null)
            {
                this._mesh_data = null;
            }
        }
    }
}